/************************************************************************
 *
 * Constants and vars
 *
 ************************************************************************/
//~ nsIContentPolicy
//~ const REJECT_REQUEST            = Components.interfaces.nsIContentPolicy.REJECT_REQUEST;
//~ const REJECT_TYPE               = Components.interfaces.nsIContentPolicy.REJECT_TYPE;
//~ const REJECT_SERVER             = Components.interfaces.nsIContentPolicy.REJECT_SERVER;
//~ const REJECT_OTHER              = Components.interfaces.nsIContentPolicy.REJECT_OTHER;
const REJECT                    = Components.interfaces.nsIContentPolicy.REJECT_REQUEST;
const ACCEPT                    = Components.interfaces.nsIContentPolicy.ACCEPT;
const TYPE_IMAGE                = Components.interfaces.nsIContentPolicy.TYPE_IMAGE;
const TYPE_OBJECT               = Components.interfaces.nsIContentPolicy.TYPE_OBJECT;

const nsIDOMDocument            = Components.interfaces.nsIDOMDocument;
const nsIDOMNode                = Components.interfaces.nsIDOMNode;

//~ cache stuff

const CACHESERVICE              = Components.classes["@mozilla.org/network/cache-service;1"]
                                            .getService(Components.interfaces.nsICacheService);

const nsICache                  = Components.interfaces.nsICache;
const nsICacheEntryDescriptor   = Components.interfaces.nsICacheEntryDescriptor;

var httpCacheSession            = CACHESERVICE.createSession("HTTP", nsICache.STORE_ANYWHERE, true);

httpCacheSession.doomEntriesIfExpired = false;

/**
    Note: During Gecko 1.8 development, nsIPrefBranchInternal was renamed to
    nsIPrefBranch2 [https://bugzilla.mozilla.org/show_bug.cgi?id=281414] and
    was frozen. nsIPrefBranchInternal name is still supported in Gecko 1.8,
    so this is what you should use in extensions that need to be compatible
    with Gecko 1.7 and Gecko 1.8 (Firefox 1.0/1.1).
    For newer extensions use nsIPrefBranch2.
**/

/************************************************************************
 *
 * Smthng...
 *
 ************************************************************************/

function nsImgLikeOpera() {
    Components.classes["@mozilla.org/observer-service;1"]
              .getService(Components.interfaces.nsIObserverService)
              .addObserver(this, "profile-after-change", false);
    
    //~ Components.classes["@mozilla.org/observer-service;1"]
              //~ .getService(Components.interfaces.nsIObserverService)
              //~ .addObserver(this, "http-on-examine-response", false);
    
    //~ this.debug = true;
}

/**
 * JS XPCOM component registration
 */
nsImgLikeOpera.prototype = {
  
  log: function(msg) {
    if (!this.debug) return;
    msg = "[ILODebugMessage]: " + msg + "\n";
    this.console.logStringMessage(msg);
    dump(msg);
  },
  
  console:        Components.classes["@mozilla.org/consoleservice;1"]
                            .getService(Components.interfaces.nsIConsoleService),
  
  prefBranch:     Components.classes["@mozilla.org/preferences-service;1"]
                            .getService(Components.interfaces.nsIPrefService)
                            .getBranch("imglikeopera."),
  
  windowMediator: Components.classes["@mozilla.org/appshell/window-mediator;1"]
                            .getService(Components.interfaces.nsIWindowMediator),
  
  settings: {
    expirationTime: 0,
    defaultPolicy: 3,
    tabsRefresh: false,
    policyPattrens: '',
    flashBlock: false,
    loadLinkedImages: 0,
    delayLoad: 0,
  },
  
  localizingDescription: function() {
    //~ localizing initial description here :o)
    //~ "Goofy-method", thanks Luana aka MatrixIsAllOver (extenzilla.it)
    
    var iloDescription   = Components.classes["@mozilla.org/supports-string;1"]
                                     .createInstance(Components.interfaces.nsISupportsString);
    
    iloDescription.data  = Components.classes["@mozilla.org/intl/stringbundle;1"]
                                     .createInstance(Components.interfaces.nsIStringBundleService)
                                     .createBundle("chrome://imglikeopera/locale/imglikeopera.properties")
                                     .GetStringFromName("initialDescription");
    
    var iloL18Prefs      = Components.classes["@mozilla.org/preferences-service;1"]
                                     .getService(Components.interfaces.nsIPrefService)
                                     .getBranch("extensions.imglikeopera@imfo.ru.")
                                     .setComplexValue("description",
                                                      Components.interfaces.nsISupportsString,
                                                      iloDescription);
  },
  
  /**
   * UserCSS
   */
  getUserCssFile: function() {
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
                         .getService(Components.interfaces.nsIProperties)
                         .get("ProfD", Components.interfaces.nsIFile);

    file.append("chrome");
    //~ "/chrome" doesn't exist when user set some folder NOT in the "..\Documents and Settings\.."
    if (!file.exists()) file.create(0x01, 0664);
    
    file.append("ilo.user.sheet.css");
    
    return file;
  },
  
  readUserCssRules: function(aCount) {
    aCount.value = 2;
    
    var fileContents = "";
    
    var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
                       .createInstance( Components.interfaces.nsIFileInputStream);
    is.init(this.getUserCssFile(), 0x01, 0, is.CLOSE_ON_EOF);
    var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
                        .createInstance(Components.interfaces.nsIScriptableInputStream);
    sis.init(is);
    
    while(sis.available() > 0)
      fileContents += sis.read(sis.available());
    
    is.close();
    sis.close();
    
    var match, rules=[];
    var reFFamily = new RegExp("family:\"(.*)\" !important", "i");
    var reFSize = new RegExp("size:(.*)px !important", "i");
    
    match = reFFamily.exec(fileContents);
    var fontfamily = (match == null) ? 'inherit' : match[1];
    
    match = reFSize.exec(fileContents);
    var fontsize = (match == null) ? 'inherit' : match[1];
    
    return [fontfamily, fontsize];
  },
  
  rewriteUserCss: function(fontfamily, fontsize) {
    fontfamily = fontfamily ? '"' + fontfamily + '"' : 'inherit';
    fontsize = fontsize ? fontsize + 'px' : 'inherit';
    
    var fileContents = "/* ImgLikeOpera bonus for userChrome.css */\n" +
                       "*[ilo-ph-fix] {\n" +
                       "-moz-force-broken-image-icon:1 !important;\n" +
                       "font-family:" + fontfamily + " !important;\n" +
                       "font-size:" + fontsize + " !important;\n" +
                       "font-weight:normal !important;\n" +
                       "line-height:140% !important;\n" +
                       "white-space:normal !important;\n" +
                       "}\n" +
                       "*[ilo-ph-fix='tofix'] {\n" +
                       "-moz-force-broken-image-icon:0 !important;\n" +
                       "border:1px inset !important;\n" +
                       "padding:1px 2px !important;\n" +
                       "white-space:pre !important;\n" +
                       "}\n" +
                       "embed.sIFR-flash[ilo-full-src] {\n" +
                       "-moz-binding: url('chrome://imglikeopera/content/imglikeopera.xml#sifr-alternate-restore') !important;\n" +
                       "}\n" +
                       "embed.sIFR-flash[ilo-ph-fix] {\n" +
                       "-moz-binding: url('chrome://imglikeopera/content/imglikeopera.xml#sifr-alternate') !important;\n" +
                       "}\n" +
                       "*[ilo-ph-fix='active'] {\n" +
                       "-moz-binding: url('chrome://imglikeopera/content/imglikeopera.xml#active-content') !important;\n" +
                       "outline-width:1px !important; outline-style:inset !important;\n" +
                       "}\n";
    
    var os = Components.classes["@mozilla.org/network/file-output-stream;1"]
                       .createInstance(Components.interfaces.nsIFileOutputStream);
    os.init(this.getUserCssFile(), 0x02 | 0x08 | 0x20, 0664, 0);
    var result = os.write(fileContents, fileContents.length);
    os.close();
    
    this.loadUserCss(); //~ reload
  },
  
  loadUserCss: function() {
    var file = this.getUserCssFile();
    
    if (!file.exists())
      return this.rewriteUserCss();
    
    var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"]
                        .getService(Components.interfaces.nsIStyleSheetService);
    var ios = Components.classes["@mozilla.org/network/io-service;1"]
                        .getService(Components.interfaces.nsIIOService);
    var u = ios.newFileURI(file);
    
    //~ reload [but not worked?!..]
    if (sss.sheetRegistered(u, sss.USER_SHEET))
      sss.unregisterSheet(u, sss.USER_SHEET);
    
    sss.loadAndRegisterSheet(u, sss.USER_SHEET);
  },
  
  loadSettings: function() {
    try {
      this.settings.expirationTime   = this.prefBranch.getIntPref("expiration.time");
      this.settings.defaultPolicy    = this.prefBranch.getIntPref("default.policy");
      this.settings.tabsRefresh      = this.prefBranch.getBoolPref("tabs.refresh");
      this.settings.flashBlock       = this.prefBranch.getBoolPref("flash.block");
      this.settings.loadLinkedImages = this.prefBranch.getIntPref("load.linked.images");
      this.settings.delayLoad        = this.prefBranch.getIntPref("delay.load");
      this.settings.policyPattrens   = this.createPatternsArray(this.prefBranch.getCharPref("policy.patterns"));
    } catch(ex) {}
  },
  
  /********************************************************************
  
  types: {
    "image": new RegExp("^image/", "i"),
    "flash": new RegExp("^application/x-shockwave-flash$", "i")
  },
    //~ image/... (png, gif, jpeg, etc.)
    //~ application/x-shockwave-flash
    //~ application/x-unknown-content-type
    
            //~ case "image/gif":
            //~ case "image/jpeg":
            //~ case "image/png":
            
            //~ do_GetService("@mozilla.org/image/loader;1")
            //~ SupportImageWithMimeType
            

  //~ setContentType: function(uri) {
    //~ just test now
    //~ this.log(uri.substr(uri.length-3,3));
  //~ },
  
  onExamineResponse: function (oHttp) {
      
      if (this.settings.expirationTime == 0) return;
      
      if ("application/x-unknown-content-type" == oHttp.contentType) {
        this.log("unknown-content-type:" + "\r\nURI: " + oHttp.URI.asciiSpec + "\r\nCType: " + oHttp.contentType);
        return;
      }
      
      //~ if ("application/x-unknown-content-type" == oHttp.contentType)
        //~ oHttp.contentType = this.setContentType(oHttp.URI.asciiSpec);
      
      for each (var type in this.types) {
        if (type.test(oHttp.contentType)) {
          oHttp.setResponseHeader("Expires", null, false);
          oHttp.setResponseHeader("Cache-Control", null, false);
          
          oHttp.setResponseHeader("Last-Modified", "Mon, 26 Jul 1997 23:23:23 GMT", false);
          oHttp.setResponseHeader("Expires", "Tue, 17 Jan 2006 23:23:23 GMT", false);
          oHttp.setResponseHeader("Cache-Control", "public, max-age=25920000", false);
          
          this.log("set:" + "\r\nURI: " + oHttp.URI.asciiSpec + "\r\nCType: " + oHttp.contentType);
          break;
        }
      }
      return;
   },
  ********************************************************************/
  
  
  observe: function(aSubject, aTopic, aData) {
    switch (aTopic) {
      //~ case "http-on-examine-response":
        //~ aSubject.QueryInterface(Components.interfaces.nsIHttpChannel);
        //~ this.onExamineResponse(aSubject);
        //~ break;
      
      case "profile-after-change":
        this.localizingDescription();
        this.loadUserCss();
        /**________________________________
       remove observer!
       **/
        break;
      
      case "app-startup":
        this.log("Hello! (ILOComponent from 'app-startup'-Observer)");
        var prefInternal = Components.classes["@mozilla.org/preferences-service;1"]
                                     .getService(Components.interfaces.nsIPrefBranch2);
        prefInternal.addObserver("imglikeopera", this, true);
      
      default:
        this.loadSettings();
    }
  },
  
  /**
   * Policy (shouldProcess/shouldLoad)
   */
  shouldProcess: function(contentType, contentLocation, context, obj) {
    return ACCEPT;
  },
  
  /**
   * So... should load?
   */
  shouldLoad: function(contentType, contentLocation, context, obj) {
    if (contentLocation.scheme != "http" && contentLocation.scheme != "https")
      return ACCEPT;
    
    /**
     * TYPE_IMAGE: <img/>, <object type="image.."/>, cssBackgrounds...
     * TYPE_OBJECT: flash... (and?..)
     */
    if (contentType != TYPE_IMAGE && (contentType != TYPE_OBJECT || !this.settings.flashBlock))
      return ACCEPT;
    
    var doc, fromCss = false;
    
    /**
     * A lot of "2DO" here...
     **/
    
    try { // <img/>
      //~ we're can't use just ".top" (aka "Ya.Mail workaround")
      //~ see http://forum.mozilla.ru/viewtopic.php?pid=67018#p67018
      //~ doc = obj.ownerDocument.defaultView.top.document;
      doc = obj.ownerDocument.defaultView.document;
    } catch(e) {
      try { // cssBackground
        //~ doc = obj.QueryInterface(nsIDOMDocument).defaultView.top.document;
        doc = obj.QueryInterface(nsIDOMDocument).defaultView.document;
        //~ this.log("cssBackground");
        //~ try { this.log("css: " + obj.QueryInterface(nsIDOMDocument).documentElement); } catch(ee){}
        fromCss = true;
      } catch(e) { // <object/>
        try {
          //~ doc = obj.QueryInterface(nsIDOMNode).ownerDocument.defaultView.top.document;
          doc = obj.QueryInterface(nsIDOMNode).ownerDocument.defaultView.document;
          //~ this.log("<object/>");
          //~ try { this.log("obj: " + obj.QueryInterface(nsIDOMNode)); } catch(ee){}
          fromCss = false;
        } catch(e) {
          try {
            doc = obj.QueryInterface(nsIDOMDocument).documentElement.ownerDocument;
            fromCss = true;
          } catch(e) {
            this.log("Catching document object... Ugggh... Something wrong.... Maybe favicon, but who knows... Object URL: " + contentLocation.spec);
            return ACCEPT;
          }
        }
      }
    }
    
    while (doc.defaultView && doc.defaultView.frameElement)
      doc = doc.defaultView.frameElement.ownerDocument;
    
    doc = new XPCNativeWrapper(doc).wrappedJSObject;
    
    /** ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ **/
    
    if (!doc.iloDocPolicy) {
      var tabs, enumerator = this.windowMediator.getEnumerator("navigator:browser");
      while (enumerator.hasMoreElements() && !doc.iloDocPolicy) {
        tabs = enumerator.getNext().window.getBrowser().browsers;
        for (var i = tabs.length; --i > -1;) {
          if (tabs[i].contentDocument && doc == tabs[i].contentDocument) {
            doc.iloDocPolicy = tabs[i].iloTabPolicy;
            doc.iloUrlsBlock = doc.iloUrlsPass = "";
            
            if (this.settings.delayLoad && doc.iloDocPolicy != 3 && doc.iloDocPolicy != 4) {
              doc.iloDocDelayLoad = doc.iloDocPolicy;
              doc.iloDocPolicy = 4;
              
              eval('\
              doc.defaultView.addEventListener("load", function(){\
                this.setTimeout("(function(){\
                                   var refreshEvent = this.document.createEvent(\'UIEvents\');\
                                   refreshEvent.initEvent(\'delayLoadRefresher\', true, false);\
                                   this.document.dispatchEvent(refreshEvent);})()"\
                , ' + this.settings.delayLoad + ')}, false);\
              ');
            }
            
            break;
          }
        }
      }
    }
    
    if (!doc.iloDocPolicy) {
      //~ this.log("Catching document object... Ugggh... Something wrong.");
      return ACCEPT; //~ smthng wrng
    }
    
    var docPolicy = doc.iloDocPolicy;
    
    if (!fromCss) {
      if (obj.getAttribute("ilo-ph-fix"))
        return obj.getAttribute("ilo-ph-fix") == "mustLoad" ?
                                                 this.passObj(obj) :
                                                 this.blockObj(obj, contentType);
      
      //~ for all non-css objects:
      obj.setAttribute("ilo-full-src", contentLocation.spec);
    }
    
    if (doc.iloDocDelayLoad) {
      if (!fromCss) obj.setAttribute("ilo-ph-fix", "delay");
      return REJECT;
    }
    
    if (docPolicy == 3 && !fromCss && this.settings.loadLinkedImages && obj.parentNode.tagName == "A") {
      //~ Load linked images in "Load cached images" mode
      docPolicy = this.settings.loadLinkedImages == 1 ?
                      1 : this.testHostEq(doc.location.host, obj.parentNode.host) ?
                      1 : docPolicy;
    } else {
      var re = new RegExp(contentLocation.spec.replace(/(\.|\||\+|\?|\$|\(|\)|\[|\])/g,"\\\$1")+" ");
      if (re.test(doc.iloUrlsBlock))
        return fromCss ? REJECT : this.blockObj(obj, contentType);
      else if (re.test(doc.iloUrlsPass))
        return fromCss ? ACCEPT : this.passObj(obj);
    }
    
    //~ check requested URL in USER filters
    docPolicy = this.getPolicyFromFilters(doc.location, contentLocation.spec) || docPolicy;
    
    //~ "Don't load images" => skip work with cache, return false.
    if (docPolicy == 4)
      return fromCss ? REJECT : this.blockObj(obj, contentType);
    
    var expire = this.checkUrlInCache(contentLocation.spec, docPolicy);
    var passIt = true;
    
    switch (docPolicy) {
      case 2:
        passIt = this.testHostEq(doc.location.host, contentLocation.host);
        break;
      
      case 3:
        passIt = expire ? false : true;
        break;
    }
    
    switch (passIt) {
      case false:
        doc.iloUrlsBlock += contentLocation.spec + " ";
        return fromCss ? REJECT : this.blockObj(obj, contentType);
        break;
      
      case true:
        doc.iloUrlsPass += contentLocation.spec + " ";
        return fromCss ? ACCEPT : this.passObj(obj);
        break;
    }
    
    //~ problems? => always return true
    return ACCEPT;
  },
  
  testHostEq: function(host1, host2) {
    host1 = host1.split(/\./);
    host2 = host2.split(/\./);
    return (host1.pop() == host2.pop() && host1.pop() == host2.pop());
  },
  
  blockObj: function(obj, contentType) {
    
    if (contentType == TYPE_OBJECT) {
      obj.setAttribute("ilo-ph-fix", "active");
      return REJECT;
    }
    
    //~ It'll be just "Submit Query" button
    if (obj.tagName.toLowerCase() != "input") {
      
      if (obj.getAttribute("alt") == null)
        obj.setAttribute("alt", "[Image]");
      
      var oW = parseInt(obj.width || obj.style.width);
      var oH = parseInt(obj.height || obj.style.height);
      
      //~ set placeholder size
      if (!oW && !oH) { obj.setAttribute("ilo-ph-fix", "tofix"); return REJECT; }
      else if (oW && !oH) { obj.iloFixSH = true; obj.height = parseInt(oW / 0.668); }
      else if (!oW && oH) { obj.iloFixSW = true; obj.width = parseInt(oH * 0.668); }
    }
    
    obj.setAttribute("ilo-ph-fix", "fixed");
    
    return REJECT;
  },
  
  passObj: function(obj) {
    obj.removeAttribute("ilo-ph-fix");
    if (obj.iloFixSH) { obj.iloFixSH = null; obj.removeAttribute("height"); }
    if (obj.iloFixSW) { obj.iloFixSW = null; obj.removeAttribute("width"); }
    return ACCEPT;
  },
  
  /**
  checkUrlInCache___NEW: function(url, docpolicy) {
    var expire = true;
    var timenow = parseInt((new Date().getTime())/1000-1);
    
    if (docpolicy == 3) {
      try {
        var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, nsICache.ACCESS_READ, false);
        if (cacheEntryDescriptor.expirationTime > timenow)
          expire = false;
      } catch(ex) {}
    }
    
    return expire;
  },**/
  
  
  checkUrlInCache: function(url, docpolicy) {
    var expire = true;
    var timenow = parseInt((new Date().getTime())/1000-1);
    
    this.log(this.settings.expirationTime);
    
    if (this.settings.expirationTime != 0) {
      try {
        var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, nsICacheEntryDescriptor, false);
        var expirationTime = cacheEntryDescriptor.lastModified + this.settings.expirationTime;
        if (expirationTime < timenow) {
          cacheEntryDescriptor.doom();
        } else if (cacheEntryDescriptor.expirationTime != expirationTime) {
          cacheEntryDescriptor.setExpirationTime(expirationTime);
          cacheEntryDescriptor.markValid();
          expire = false;
        }
        cacheEntryDescriptor.close();
        /************************************************************************
         *
         * "Я не знаю кому и зачем это нужно" (c) Вертинский
         *
         * Тут такая тема... Если повторно не обратиться сейчас к кэшу, то
         * "setExpirationTime" (см. выше) не сработает, что всплывает при
         * перезагрузке страницы.
         *
         * 2Do: Надо бы разобраться и поковырять.
         * Позже: во всяком случае, так было в FF 0.9.3
         * Ещё позже: ff1.0 -- ага, так и есть
         *
         ************************************************************************/
         try {
           cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, nsICache.ACCESS_READ, false);
           if (cacheEntryDescriptor)
             expire = false;
         } catch(ex) {}
            
      } catch(ex) {}
    
    } else if (docpolicy == 3) {
      try {
        var cacheEntryDescriptor = httpCacheSession.openCacheEntry(url, nsICache.ACCESS_READ, false);
        if (cacheEntryDescriptor.expirationTime > timenow)
          expire = false;
      } catch(ex) {}
    }
    
    return expire;
  },
  
  getPolicyFromFilters: function(docLocation, url) {
    var policy, patts = this.settings.policyPattrens;
    for (var i=0; i<patts.length; i++) {
      if (patts[i][0].test(docLocation) || patts[i][0].test(url)) {
        policy = patts[i][1];
        break;
      }
    }
    return policy;
  },
  
  createPatternsArray: function(str) {
    var patt, pattern = [], tmp = str.split("  ");
    var tmpCount = tmp.length-1;
    for (var i=0; i<tmpCount; i++) {
      patt = tmp[i].split(" ");
      patt[1] = patt[1][0]-0;
      if (patt.length == 2 && patt[1]>0 && patt[1]<5)
        pattern.push([this.txt2RegExp(patt[0]), patt[1]]);
    }
    return pattern;
  },
  
  txt2RegExp: function(txt) {
    
    //~ not simple; return power
    if (txt[0] == "/" && txt[2]) {
      var mod = "", txtEnd = txt.substr(txt.length-2, txt.length);
      
      if (txtEnd == "/i" || txtEnd == "/I") { mod = "i"; var res = txt.substr(1, txt.length-3); }
      else if (txtEnd[1] == "/") { var res = txt.substr(1, txt.length-2); }
      
      try { var reg = new RegExp(res, mod); } catch(e){}
      
      if (reg) return reg;
    }
    
    //~ else: simple;
    
    var res = txt.replace(/^[\*]*(.(?:[^\*]|[\*]+[^\*])*)[\*]*$/, "$1").replace(/[\*]+/, "*").replace(/([^\w\*])/g, "\\$1").replace(/\*/g, ".*") + "(?:\\n)?";
    
    return new RegExp(res, 'i');
  },


  QueryInterface: function(iid) {
    if (!iid.equals(Components.interfaces.nsIImgLikeOpera) &&
        !iid.equals(Components.interfaces.nsISupports) &&
        !iid.equals(Components.interfaces.nsISupportsWeakReference) &&
        !iid.equals(Components.interfaces.nsIContentPolicy) &&
        !iid.equals(Components.interfaces.nsIObserver))
    {
      dump("nsImgLikeOpera: QI unknown interface: " + iid + "\n");
      throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  },
  
  /**
   * Component's module implementation
   */
  module: {
    // registerSelf: Register this component.
    registerSelf: function (compMgr, fileSpec, location, type) {
      var compReg = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
      compReg.registerFactoryLocation(this.cid,
                                      "nsImgLikeOpera JS component",
                                      this.contractId,
                                      fileSpec, location, type );
      var catman = Components.classes["@mozilla.org/categorymanager;1"]
                             .getService(Components.interfaces.nsICategoryManager);
      catman.addCategoryEntry("app-startup", "nsImgLikeOpera", this.contractId, true, true);
      catman.addCategoryEntry("content-policy", this.contractId, this.contractId,true,true);
    },
    
    unregisterSelf: function(compMgr, fileSpec, location) {
      compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
      compMgr.unregisterFactoryLocation(this.cid, fileSpec);
      var catman = Components.classes["@mozilla.org/categorymanager;1"]
                             .getService(Components.interfaces.nsICategoryManager);
      catman.deleteCategoryEntry("app-startup", this.contractId, true);
      catman.deleteCategoryEntry("content-policy", this.contractId, true);
    },
    
    // getClassObject: Return this component's factory object.
    getClassObject: function (compMgr, cid, iid) {
      if (!cid.equals(this.cid))
        throw Components.results.NS_ERROR_NO_INTERFACE;
      if (!iid.equals(Components.interfaces.nsIFactory))
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
      return this.factory;
    },
    
    /* CID for this class */
    // extensions.roachfiend.com/cgi-bin/guid.pl
    cid: Components.ID("{9aa46f4f-4dc7-4c06-97af-5035170633fe}"),
    
    /* Contract ID for this class */
    contractId: "@mozilla.org/imglikeopera;1",
    
    /* factory object */
    factory: {
      // createInstance: Return a new nsImgLikeOpera object.
      createInstance: function (outer, iid) {
        if (outer != null)
          throw Components.results.NS_ERROR_NO_AGGREGATION;
        return new nsImgLikeOpera();
      }
    },
    
    canUnload: function(compMgr) {
      return true;
    }
  }
}

// NSGetModule: Return the nsIModule object.
function NSGetModule(compMgr, fileSpec) {
    return nsImgLikeOpera.prototype.module;
}